% Read/Write StagYY data
%
% Read binary output files of STAG3D and write them into Paraview files
%
% Boris Kaus, 26.2.2008
%
% PJT edits August 2010:
% - spherical: move coordinate transform just before writing, otherwise reset
% - spherical: centre domain around equator, rotate velocity components
% - YinYang: transform velocity components
% - new Annulus mode (replaced previous 'Cylindrical' mode)

% PJT edits July 2015: Added stress and strain rate
% PJT additions January 2021: 
% - Basalt, HPE, conductivity, dislocation creep & plasticity fields
% - The ability to go through multiple cases/directories
% - Skips reading/writing for frames where vtk file already exists
%    (but still writes pvd file and reads T files to get the time)

clear

n=1;   % List multiple output directories below, always with n=n+1 after
directorylist(n) =   "/Users/pjt/StagRuns/Case1/+op"; n=n+1;
directorylist(n) =   "/Users/pjt/StagRuns/Case2/+op"; n=n+1;
directorylist(n) =   "/Users/pjt/StagRuns/Case3/+op"; n=n+1;
directorylist(n) =   "/Users/pjt/StagRuns/Case4/+op"; n=n+1;
directorylist(n) =   "/Users/pjt/StagRuns/Case5/+op"; n=n+1;

fname           =   'earthDIM';       % file stem - same for all cases

GridType        =   'YinYang';         % choose one of these geometries
%GridType        =   'Spherical';
%GridType        =   'Annulus';
%GridType        =   'Cartesian';


% Specify which fields we want to read/write in addition to T:
WriteViscosity  =   true; 
WriteVelocity   =   false;
WriteTCond      =   false;
WriteComposition=   false;
WriteBasalt     =   false;
WriteHPE        =   false;
WriteMeltFrac   =   false;
WriteStress     =   false;
WriteStrainRate =   false;
WriteDislCreep  =   false;
WritePlasticity =   false;
WriteDensity    =   false;
WriteWater      =   false;
WriteImpactor   =   false;
WriteMetal      =   false;
WriteTraPerCell =   false;

% Specify start and end frames, and step
number_start = 0;
number_end = 999;
step = 1;

for idir = 1:length(directorylist)
    directory = directorylist(idir)

    num = 1; % number of frame for name array

    fname_number=number_start;
    while fname_number <= number_end
        fprintf(1,'Frame number = %d : ',fname_number)
        if fname_number<10000
            number_string = num2str(10000+fname_number);
            number_string(1)='0';
        else
            number_string = num2str(fname_number);
        end
        fname_T = strcat(directory,'/',fname,'_t',number_string);

        switch GridType
            case 'YinYang'
                % YING YANG GRID

                if exist(fname_T)==0
                    % The file does not exist => we have finished processing all data

                    disp(['File does not exist => finished processing all files'])
                    fname_number = number_end + 1;


                else
                    
                    % Check whether vtk file already exists
                    fname_vtk = strcat(directory,'/',fname,'_',num2str(1000000+fname_number),'.vtu');

                    if exist(fname_vtk)==2
                        
                        disp(['vtk file already exists'])
                        fname_vtk = strcat(fname,'_',num2str(1000000+fname_number),'.vtu'); % without the directory
                        time = ReadStagTime(fname_T);

                    else % does not exist - read fields and create

                        % Read temperature information & time
                        [nnb, X_3D, Y_3D, Z_3D,T_3D_1, T_3D_2, time, rcmb] = ReadStag3D2(directory, fname, fname_number, 'temperature');

                        if WriteVelocity
                            % Read pressure & velocity information
                            [nnb, X_3D, Y_3D, Z_3D,VX_3D_1, VY_3D_1, VZ_3D_1, P_3D_1, VX_3D_2, VY_3D_2, VZ_3D_2, P_3D_2, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                            if nnb==-999
                                error('Velocity file not found')
                            end
                        end

                        if WriteViscosity
                            % Read viscosity information
                            [nnb, X_3D, Y_3D, Z_3D,ETA_3D_1, ETA_3D_2, time, rcmb] = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                            if nnb==-999
                                error('Viscosity file not found')
                            end
                        end

                        if WriteComposition
                            % Read viscosity information
                            [nnb, X_3D, Y_3D, Z_3D,C_3D_1, C_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'composition');
                            if nnb==-999
                                error('Composition file not found')
                            end
                        end

                        if WriteMeltFrac
                            % Read viscosity information
                            [nnb, X_3D, Y_3D, Z_3D,F_3D_1, F_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                            if nnb==-999
                                error('Melt fraction file not found')
                            end
                        end

                        if WriteStress
                            % Read stress information
                            [nnb, X_3D, Y_3D, Z_3D,S_3D_1, S_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'stress');
                            if nnb==-999
                                error('Stress file not found')
                            end
                        end

                        if WriteStrainRate
                            % Read strain rate information
                            [nnb, X_3D, Y_3D, Z_3D,E_3D_1, E_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                            if nnb==-999
                                error('Strain rate file not found')
                            end
                        end

                        if WriteBasalt
                            % Read basalt fraction information
                            [nnb,X_3D, Y_3D, Z_3D,BS_3D_1, BS_3D_2, time]          = ReadStag3D2(directory, fname, fname_number, 'basalt');
                            if nnb==-999
                                error('Basalt file not found')
                            end
                        end

                        if WriteDensity
                            % Read density information
                            [nnb,X_3D, Y_3D, Z_3D,DEN_3D_1, DEN_3D_2, time]          = ReadStag3D2(directory, fname, fname_number, 'density');
                            if nnb==-999
                                error('Density file not found')
                            end
                        end

                        if WriteHPE
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,HPE_3D_1,HPE_3D_2, time]         = ReadStag3D2(directory, fname, fname_number, 'HPE');
                            if nnb==-999
                                error('HPE file not found')
                            end
                        end

                        if WriteTraPerCell
                            % Read Tracers/cell information
                            [nnb,X_3D, Y_3D, Z_3D,TPC_3D_1,TPC_3D_2, time]         = ReadStag3D2(directory, fname, fname_number, 'trapercell');
                            if nnb==-999
                                error('TraPerCell file not found')
                            end
                        end

                        if WriteWater
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,W_3D_1,W_3D_2, time]             = ReadStag3D2(directory, fname, fname_number, 'water');
                            if nnb==-999
                                error('Water file not found')
                            end
                        end

                        if WriteImpactor
                            % Read Impactor information
                            [nnb,X_3D, Y_3D, Z_3D,I_3D_1,I_3D_2, time]             = ReadStag3D2(directory, fname, fname_number, 'impactor');
                            if nnb==-999
                                error('Impactor file not found')
                            end
                        end

                        if WriteMetal
                            % Read Metal information
                            [nnb,X_3D, Y_3D, Z_3D,M_3D_1,M_3D_2, time]             = ReadStag3D2(directory, fname, fname_number, 'metal');
                            if nnb==-999
                                error('Metal file not found')
                            end
                        end

                        if WriteTCond
                            % Read thermal conductivity information
                            [nnb,X_3D, Y_3D, Z_3D,K_3D_1,K_3D_2, time]             = ReadStag3D2(directory, fname, fname_number, 'conductivity');
                            if nnb==-999
                                error('TCond file not found')
                            end
                        end

                        if WriteDislCreep
                            % Read disl. creep information
                            [nnb,X_3D, Y_3D, Z_3D,RD_3D_1,RD_3D_2, time]           = ReadStag3D2(directory, fname, fname_number, 'dislocation');
                            if nnb==-999
                                error('Dislocation creep file not found')
                            end
                        end
                        if WritePlasticity
                            % Read plasticity information
                            [nnb,X_3D, Y_3D, Z_3D,RP_3D_1,RP_3D_2, time]           = ReadStag3D2(directory, fname, fname_number, 'plasticity');
                            if nnb==-999
                                error('Plasticity file not found')
                            end
                        end

                        xs = X_3D(:,:,end)+pi/4;
                        ys = Y_3D(:,:,end)-pi/4;
                        zs = Z_3D(:,:,end);

                        % Transform coordinates for Yin & Yang grids
                        R  = Z_3D+rcmb;     lat = pi/4-X_3D;     lon = Y_3D-3*pi/4;

                        X_3D_1 = R.*cos(lat).*cos(lon);   % Yin grid
                        Y_3D_1 = R.*cos(lat).*sin(lon);
                        Z_3D_1 = R.*sin(lat);

                        X_3D_2 = -X_3D_1;       %  Yang grid
                        Y_3D_2 =  Z_3D_1;
                        Z_3D_2 =  Y_3D_1;

                        % Transform velocities, if needed
                        if WriteVelocity
                            Vtheta = VX_3D_1; Vphi = VY_3D_1; Vr = VZ_3D_1;                                 % on Yin grid
                            VX_3D_1 =  Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon);
                            VY_3D_1 =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                            VZ_3D_1 = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                            Vtheta = VX_3D_2; Vphi = VY_3D_2; Vr = VZ_3D_2;                                 % on Yang grid
                            VX_3D_2 = -( Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon) );
                            VZ_3D_2 =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                            VY_3D_2 = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                        end

                        % Write a ying-yang grid
                        WriteStag3D_VTK_YinYang2;    % Write VTK XML file

                    end
                    
                    % Store the filename and time in a structure, which is later
                    % used to create a pvd file, that contains all timesteps and
                    % times of the files
                    FileNames{num} = {time,fname_number,fname_vtk};

                end


            case 'Cartesian'
                % CARTESIAN GRID
                
                if exist(fname_T)==0
                    % The file does not exist => we have finished processing all data

                    disp(['File does not exist => finished processing all files'])
                    fname_number = number_end + 1;

                else

                    % Check whether vtk file already exists
                    fname_vtk = strcat(directory,'/',fname,'_',num2str(1000000+fname_number),'.vts');

                    if exist(fname_vtk)==2
                        
                        disp(['vtk file already exists'])
                        fname_vtk = strcat(fname,'_',num2str(1000000+fname_number),'.vts'); % without the directory
                        time = ReadStagTime(fname_T);

                    else % does not exist - read fields and create

                        % Read temperature information & time
                        [nnb, X_3D, Y_3D, Z_3D,T_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'temperature');

                        if WriteVelocity
                            % Read pressure & velocity information
                            [nnb,X_3D, Y_3D, Z_3D, VX_3D, VY_3D, VZ_3D, P_3D, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                            if nnb==-999
                                error('Velocity file not found')
                            end
                        end

                        if WriteViscosity
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,ETA_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                            if nnb==-999
                                error('Viscosity file not found')
                            end
                        end

                        if WriteComposition
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,C_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'composition');
                            if nnb==-999
                                error('Composition file not found')
                            end
                        end

                        if WriteMeltFrac
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,F_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                            if nnb==-999
                                error('Melt fraction file not found')
                            end
                        end

                        if WriteStress
                            % Read stress information
                            [nnb,X_3D, Y_3D, Z_3D,S_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'stress');
                            if nnb==-999
                                error('Stress file not found')
                            end
                        end

                        if WriteStrainRate
                            % Read strain rate information
                            [nnb,X_3D, Y_3D, Z_3D,E_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                            if nnb==-999
                                error('Strain rate file not found')
                            end
                        end

                        if WriteDensity
                            % Read density information
                            [nnb,X_3D, Y_3D, Z_3D,DEN_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'density');
                            if nnb==-999
                                error('Density file not found')
                            end
                        end
                        
                        if WriteBasalt
                            % Read basalt fraction information
                            [nnb,X_3D, Y_3D, Z_3D,BS_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'basalt');
                            if nnb==-999
                                error('Basalt file not found')
                            end
                        end

                        if WriteHPE
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,HPE_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'HPE');
                            if nnb==-999
                                error('HPE file not found')
                            end
                        end

                        if WriteWater
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,W_3D, time]      = ReadStag3D2(directory, fname, fname_number, 'water');
                            if nnb==-999
                                error('Water file not found')
                            end
                        end

                        if WriteImpactor
                            % Read Impactor Element information
                            [nnb,X_3D, Y_3D, Z_3D,I_3D, time]      = ReadStag3D2(directory, fname, fname_number, 'impactor');
                            if nnb==-999
                                error('Impactor file not found')
                            end
                        end

                        if WriteMetal
                            % Read Impactor Element information
                            [nnb,X_3D, Y_3D, Z_3D,M_3D, time]      = ReadStag3D2(directory, fname, fname_number, 'metal');
                            if nnb==-999
                                error('Metal file not found')
                            end
                        end

                        if WriteTCond
                            % Read thermal conductivity information
                            [nnb,X_3D, Y_3D, Z_3D,K_3D, time]      = ReadStag3D2(directory, fname, fname_number, 'conductivity');
                            if nnb==-999
                                error('TCond file not found')
                            end
                        end

                        if WriteDislCreep
                            % Read disl. creep information
                            [nnb,X_3D, Y_3D, Z_3D,RD_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'dislocation');
                            if nnb==-999
                                error('Dislocation creep file not found')
                            end
                        end
                        if WritePlasticity
                            % Read plasticity information
                            [nnb,X_3D, Y_3D, Z_3D,RP_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'plasticity');
                            if nnb==-999
                                error('Plasticity file not found')
                            end
                        end    

                        %WriteStag3D_LegacyVTK_Cartesian;   %-> Writes VTK legacy files
                        WriteStag3D_VTK_Cartesian;          %-> Writes VTK-XML files

                    end
                    
                    % Store the filename and time in a structure, which is later
                    % used to create a pvd file, that contains all timesteps and
                    % times of the files
                    FileNames{num} = {time,fname_number,fname_vtk};

                end

            case 'Spherical'
                % SPHERICAL GRID


                if exist(fname_T)==0
                    % The file does not exist => we have finished processing all data

                    disp(['File does not exist => finished processing all files'])
                    fname_number = number_end + 1;

                else

                    % Check whether vtk file already exists
                    fname_vtk = strcat(directory,'/',fname,'_',num2str(1000000+fname_number),'.vts');

                    if exist(fname_vtk)==2
                        
                        disp(['vtk file already exists'])
                        fname_vtk = strcat(fname,'_',num2str(1000000+fname_number),'.vts'); % without the directory
                        time = ReadStagTime(fname_T);

                    else % does not exist - read fields and create

                        % Read temperature information & time
                        [nnb, X_3D, Y_3D, Z_3D,T_3D, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'temperature');
                        
                        if WriteVelocity
                            % Read pressure & velocity information
                            [nnb,X_3D, Y_3D, Z_3D, VX_3D, VY_3D, VZ_3D, P_3D, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                            if nnb==-999
                                error('Velocity file not found')
                            end
                        end

                        if WriteViscosity
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,ETA_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                            if nnb==-999
                                error('Viscosity file not found')
                            end
                        end

                        if WriteComposition
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,C_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'composition');
                            if nnb==-999
                                error('Composition file not found')
                            end
                        end

                        if WriteMeltFrac
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,F_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                            if nnb==-999
                                error('Melt fraction file not found')
                            end
                        end

                        if WriteStress
                            % Read stress information
                            [nnb,X_3D, Y_3D, Z_3D,S_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'stress');
                            if nnb==-999
                                error('Stress file not found')
                            end
                        end

                        if WriteStrainRate
                            % Read strain rate information
                            [nnb,X_3D, Y_3D, Z_3D,E_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                            if nnb==-999
                                error('Strain rate file not found')
                            end
                        end

                        if WriteBasalt
                            % Read basalt fraction information
                            [nnb,X_3D, Y_3D, Z_3D,BS_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'basalt');
                            if nnb==-999
                                error('Basalt file not found')
                            end
                        end

                        if WriteDensity
                            % Read density information
                            [nnb,X_3D, Y_3D, Z_3D,DEN_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'density');
                            if nnb==-999
                                error('Density file not found')
                            end
                        end

                        if WriteHPE
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,HPE_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'HPE');
                            if nnb==-999
                                error('HPE file not found')
                            end
                        end

                        if WriteWater
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,W_3D, time]       = ReadStag3D2(directory, fname, fname_number, 'water');
                            if nnb==-999
                                error('Water file not found')
                            end
                        end

                        if WriteImpactor
                            % Read Impactor information
                            [nnb,X_3D, Y_3D, Z_3D,I_3D, time]       = ReadStag3D2(directory, fname, fname_number, 'impactor');
                            if nnb==-999
                                error('Impactor file not found')
                            end
                        end

                        if WriteMetal
                            % Read Metal information
                            [nnb,X_3D, Y_3D, Z_3D,M_3D, time]       = ReadStag3D2(directory, fname, fname_number, 'metal');
                            if nnb==-999
                                error('Metal file not found')
                            end
                        end

                        if WriteTCond
                            % Read thermal conductivity information
                            [nnb,X_3D, Y_3D, Z_3D,K_3D, time]      = ReadStag3D2(directory, fname, fname_number, 'conductivity');
                            if nnb==-999
                                error('TCond file not found')
                            end
                        end

                        if WriteDislCreep
                            % Read disl. creep information
                            [nnb,X_3D, Y_3D, Z_3D,RD_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'dislocation');
                            if nnb==-999
                                error('Dislocation creep file not found')
                            end
                        end
                        if WritePlasticity
                            % Read plasticity information
                            [nnb,X_3D, Y_3D, Z_3D,RP_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'plasticity');
                            if nnb==-999
                                error('Plasticity file not found')
                            end
                        end

                        % Transform coordinates for Yin grid
                        latmean = (max(max(max(X_3D))) + min(min(min(X_3D)))) / 2;       % spherical patch is centred at equator
                        lonmean = (max(max(max(Y_3D))) + min(min(min(Y_3D)))) / 2;       
                        R    = Z_3D + rcmb;  lat = latmean - X_3D;  lon = Y_3D - lonmean;
                        X_3D = R.*cos(lat).*cos(lon);
                        Y_3D = R.*cos(lat).*sin(lon);
                        Z_3D = R.*sin(lat);

                        % Transform velocities if they are needed
                        if WriteVelocity
                            Vtheta = VX_3D; Vphi = VY_3D; Vr = VZ_3D;
                            VX_3D =  Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon);
                            VY_3D =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                            VZ_3D = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                        end

                        %WriteStag3D_LegacyVTK_Cartesian;   %-> Writes VTK legacy files
                        WriteStag3D_VTK_Cartesian;          %-> Writes VTK-XML files

                    end
                    
                    % Store the filename and time in a structure, which is later
                    % used to create a pvd file, that contains all timesteps and
                    % times of the files
                    FileNames{num} = {time,fname_number,fname_vtk};

                end

            case 'Annulus'
                % SPHERICAL ANNULUS GRID
                % same as spherical but add extra points to avoid a gap in the
                % visualisation


                if exist(fname_T)==0
                    % The file does not exist => we have finished processing all data

                    disp(['File does not exist => finished processing all files'])
                    fname_number = number_end + 1;

                else

                    % Check whether vtk file already exists
                    fname_vtk = strcat(directory,'/',fname,'_',num2str(1000000+fname_number),'.vts');

                    if exist(fname_vtk)==2
                        
                        disp(['vtk file already exists'])
                        fname_vtk = strcat(fname,'_',num2str(1000000+fname_number),'.vts'); % without the directory
                        time = ReadStagTime(fname_T);

                    else % does not exist - read fields and create

                        % Read temperature information & time
                        [nnb, X_3D, Y_3D, Z_3D,T_3D, time, rcmb] = ReadStag3D2(directory, fname, fname_number, 'temperature');
                        % add one row for nicer visualization when a full annulus
                        ny = size(Y_3D,2);
                        ay = (Y_3D(1,end,1)-Y_3D(1,1,1))*ny/(ny-1);
                        full_annulus = ay>=2*pi-0.01;
                        if full_annulus
                            T_3D(1,end+1,:) = T_3D(1,1,:);
                        end


                        if WriteVelocity
                            % Read pressure & velocity information
                            [nnb,X_3D, Y_3D, Z_3D, VX_3D, VY_3D, VZ_3D, P_3D, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                            if full_annulus
                                VX_3D(1,end+1,:) = VX_3D(1,1,:);
                                VY_3D(1,end+1,:) = VY_3D(1,1,:);
                                VZ_3D(1,end+1,:) = VZ_3D(1,1,:);
                                 P_3D(1,end+1,:) =  P_3D(1,1,:);
                            end
                       end

                        if WriteViscosity
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,ETA_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                            if nnb==-999
                                error('Viscosity file not found')
                            end
                            if full_annulus
                                ETA_3D(1,end+1,:) = ETA_3D(1,1,:);
                            end
                        end

                        if WriteComposition
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,C_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'composition');
                            if nnb==-999
                                error('Composition file not found')
                            end
                            if full_annulus
                                C_3D(1,end+1,:) = C_3D(1,1,:);
                            end
                        end

                        if WriteMeltFrac
                            % Read viscosity information
                            [nnb,X_3D, Y_3D, Z_3D,F_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                            if nnb==-999
                                error('Melt fraction file not found')
                            end
                            if full_annulus
                                F_3D(1,end+1,:) = F_3D(1,1,:);
                            end
                        end

                        if WriteStress
                            % Read stress information
                            [nnb,X_3D, Y_3D, Z_3D,S_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'stress');
                            if nnb==-999
                                error('Stress file not found')
                            end
                            if full_annulus
                                S_3D(1,end+1,:) = S_3D(1,1,:);
                            end
                        end

                        if WriteStrainRate
                            % Read strain rate information
                            [nnb,X_3D, Y_3D, Z_3D,E_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                            if nnb==-999
                                error('Strain rate file not found')
                            end
                            if full_annulus
                                E_3D(1,end+1,:) = E_3D(1,1,:);
                            end
                        end

                        if WriteBasalt
                            % Read basalt fraction information
                            [nnb,X_3D, Y_3D, Z_3D,BS_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'basalt');
                            if nnb==-999
                                error('Basalt file not found')
                            end
                            if full_annulus
                                BS_3D(1,end+1,:) = BS_3D(1,1,:);
                            end
                        end

                        if WriteDensity
                            % Read density information
                            [nnb,X_3D, Y_3D, Z_3D,DEN_3D, time]    = ReadStag3D2(directory, fname, fname_number, 'density');
                            if nnb==-999
                                error('Density file not found')
                            end
                            if full_annulus
                                DEN_3D(1,end+1,:) = DEN_3D(1,1,:);
                            end
                        end

                        if WriteHPE
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,HPE_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'HPE');
                            if nnb==-999
                                error('HPE file not found')
                            end
                            if full_annulus
                                HPE_3D(1,end+1,:) = HPE_3D(1,1,:);
                            end
                        end

                        if WriteWater
                            % Read Heat-Producing Element information
                            [nnb,X_3D, Y_3D, Z_3D,W_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'water');
                            if nnb==-999
                                error('Water file not found')
                            end
                            if full_annulus
                                W_3D(1,end+1,:) = W_3D(1,1,:);
                            end
                        end

                        if WriteImpactor
                            % Read Impactor information
                            [nnb,X_3D, Y_3D, Z_3D,I_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'impactor');
                            if nnb==-999
                                error('Impactor file not found')
                            end
                            if full_annulus
                                I_3D(1,end+1,:) = I_3D(1,1,:);
                            end
                        end

                        if WriteMetal
                            % Read Metal information
                            [nnb,X_3D, Y_3D, Z_3D,M_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'metal');
                            if nnb==-999
                                error('Metal file not found')
                            end
                            if full_annulus
                                M_3D(1,end+1,:) = M_3D(1,1,:);
                            end
                        end

                        if WriteTCond
                            % Read thermal conductivity information
                            [nnb,X_3D, Y_3D, Z_3D,K_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'conductivity');
                            if nnb==-999
                                error('TCond file not found')
                            end
                            if full_annulus
                                K_3D(1,end+1,:) = K_3D(1,1,:);
                            end
                        end

                        if WriteDislCreep
                            % Read disl. creep information
                            [nnb,X_3D, Y_3D, Z_3D,RD_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'dislocation');
                            if nnb==-999
                                error('Dislocation creep file not found')
                            end
                            if full_annulus
                                RD_3D(1,end+1,:) = RD_3D(1,1,:);
                            end
                        end
                        if WritePlasticity
                            % Read plasticity information
                            [nnb,X_3D, Y_3D, Z_3D,RP_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'plasticity');
                            if nnb==-999
                                error('Plasticity file not found')
                            end
                            if full_annulus
                                RP_3D(1,end+1,:) = RP_3D(1,1,:);
                            end
                        end


                        if full_annulus
                            Y_3D(1,end+1,:) = Y_3D(1,1,:) + 2*pi;
                            X_3D(1,end+1,:) = X_3D(1,1,:);
                            Z_3D(1,end+1,:) = Z_3D(1,1,:);
                        end

                        % Transform coordinates for Yin grid
                        R    = Z_3D + rcmb;  lat = zeros(size(X_3D));
                        if full_annulus
                            lon = Y_3D-pi;         % in stagyy, phi is from -pi to +pi
                        else
                            lon = Y_3D+pi/2-ay/2;  % partial annulus - centre at top
                        end

                        X_3D = R.*cos(lat).*cos(lon);
                        Y_3D = R.*cos(lat).*sin(lon);
                        Z_3D = R.*sin(lat);

                        % Transform velocities if they are needed
                        if WriteVelocity
                            Vtheta = VX_3D; Vphi = VY_3D; Vr = VZ_3D;
                            VX_3D =  Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon);
                            VY_3D =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                            VZ_3D = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                        end

                        %WriteStag3D_LegacyVTK_Cartesian;   %-> Writes VTK legacy files
                        WriteStag3D_VTK_Cartesian;          %-> Writes VTK-XML files

                    end
                    
                    % Store the filename and time in a structure, which is later
                    % used to create a pvd file, that contains all timesteps and
                    % times of the files
                    FileNames{num} = {time,fname_number,fname_vtk};
                end

            otherwise

                error('Unknown gridtype')
                
        end

        num = num+1;
        fname_number = fname_number + step;

    end
    
    % Create a PVD file, which contains all data including time information
    disp('Creating PVD file')
    Create_PVD_file(FileNames,fname,directory);
    clear FileNames;
    disp('Finished')

end









